package mqtt

import (
	"fmt"
	"kpt-pasture/config"
	"sync"
	"time"

	"gitee.com/xuyiping_admin/pkg/logger/zaplog"
	golangMqtt "github.com/eclipse/paho.mqtt.golang"
	"go.uber.org/zap"
)

var messagePubHandler golangMqtt.MessageHandler = func(client golangMqtt.Client, msg golangMqtt.Message) {
	zaplog.Info("messagePubHandlerReceived", zap.Any("message", string(msg.Payload())), zap.Any("topic", msg.Topic()))
}

var connectHandler golangMqtt.OnConnectHandler = func(client golangMqtt.Client) {
	zaplog.Info("connectedClient", zap.Any("client", client))
}

var connectLostHandler golangMqtt.ConnectionLostHandler = func(client golangMqtt.Client, err error) {
	zaplog.Info("connectLost", zap.Any("err", err.Error()))
}

func (d *DataEventEntry) NewMqtt(conf config.MqttSetting) golangMqtt.Client {
	opts := golangMqtt.NewClientOptions()
	opts.AddBroker(fmt.Sprintf("tcp://%s:%d", conf.Broker, conf.Port))
	opts.SetClientID(conf.ClientId)
	opts.SetCleanSession(false)
	opts.SetUsername(conf.UserName)
	opts.SetPassword(conf.Password)
	opts.SetAutoReconnect(conf.AutoReconnect)
	opts.SetDefaultPublishHandler(messagePubHandler)
	opts.OnConnect = connectHandler
	opts.OnConnectionLost = connectLostHandler
	client := golangMqtt.NewClient(opts)
	if token := client.Connect(); token.Wait() && token.Error() != nil {
		panic(token.Error())
	}
	return client
}

var bufferPool = sync.Pool{
	New: func() interface{} {
		return make([]byte, 1024) // 根据实际情况调整缓冲区大小
	},
}

func (d *DataEventEntry) SubMsg(conf config.MqttSetting, client golangMqtt.Client) {
	var subMsgChan = make(chan []byte, 2*conf.WorkNumber)
	if token := client.Subscribe(conf.Topic, byte(conf.Qos), func(client golangMqtt.Client, msg golangMqtt.Message) {
		buffer := bufferPool.Get().([]byte)
		copy(buffer, msg.Payload())
		subMsgChan <- buffer[:len(msg.Payload())]
	}); token.Wait() && token.Error() != nil {
		close(subMsgChan)
		zaplog.Error("SubMsg", zap.Any("configOption", conf), zap.Any("err", token.Error()))
		return
	}

	defer close(subMsgChan)
	for i := 0; i < conf.WorkNumber; i++ {
		go func() {
			ticker := time.NewTicker(10 * time.Second) // 根据实际情况调整超时时间
			defer ticker.Stop()
			for {
				select {
				case msg := <-subMsgChan:
					d.ProcessMessages(msg)
					bufferPool.Put(msg)
				case <-ticker.C:
					zaplog.Info("subMsgChan timeout")
					return
				}
			}
		}()
	}
}

func (d *DataEventEntry) ProcessMessages(msg []byte) {

}
